@@ -685,6 +685,7 @@ patternResized(Widget w, LocalInfo * l, XConfigureEvent * event, Boolean * flg)
 {
     Dimension width, height;
     Dimension height1, height2;
+
     XFlush(XtDisplay(l->form));   
     XtVaGetValues(XtParent(l->form), XtNwidth, &width, 
 		                     XtNheight, &height, NULL);
@@ -692,7 +693,11 @@ patternResized(Widget w, LocalInfo * l, XConfigureEvent * event, Boolean * flg)
     if (height<300) height = 300;
 #ifdef XAW3D
     height1 = ((height-61)*265)/480;
-    if (height1<268) height1 = 268;
+#ifdef XAW3DG
+    if (height1<271) height1 = 271;
+#else
+    if (height1<268) height1 = 268;   
+#endif   
     height2 = ((height-61)*208)/480;
 #else   
     height1 = ((height-59)*276)/480;
@@ -701,6 +706,10 @@ patternResized(Widget w, LocalInfo * l, XConfigureEvent * event, Boolean * flg)
 #endif   
     XtResizeWidget(l->form, width-8, height1, 1);
     XtMoveWidget(l->form, 4, 54);
+    XtResizeWidget(l->form1, width-8, height2, 1);
+    XtMoveWidget(l->form1, 4, 60 + height1);      
+    XtResizeWidget(l->form2, width-14, height2-8, 0);
+    XtMoveWidget(l->form2, 4, 4);   
 #ifdef XAWPLAIN
     XtResizeWidget(l->vport, width-345, height1-32, 0);   
     XtResizeWidget(l->box, width-345, height1-32, 0);
@@ -713,14 +722,12 @@ patternResized(Widget w, LocalInfo * l, XConfigureEvent * event, Boolean * flg)
     XtResizeWidget(l->vport, width-345, height1-32, 0);
     XtResizeWidget(l->box, width-348, height1-35, 0);   
 #endif
-    XtResizeWidget(l->form1, width-8, height2, 1);
-    XtMoveWidget(l->form1, 4, 60 + height1);   
-    XtResizeWidget(l->form2, width-16, height2-8, 0);
-    XtMoveWidget(l->form2, 4, 4);   
-    XtResizeWidget(l->vport2, width-16, height2-8, 0);
-    XtResizeWidget(l->patternbox, width-16, height2-8, 0);
+    XtResizeWidget(l->patternbox, width-16, height2-8, 0);   
     XtVaSetValues(l->patternbox, XtNwidth, width-16, XtNheight, height2-8, 
 		  NULL);
+    XtResizeWidget(l->vport2, width-16, height2-8, 0);   
+    XtVaSetValues(l->vport2, XtNwidth, width-16, XtNheight, height2-8, 
+		  NULL);
     XtMoveWidget(l->vport2, 0, 0);
 }
 
@@ -780,6 +787,7 @@ deletePatternCB(Widget mb, Widget w)
     PatternInfo *info, *ainfo;
     Widget parent;
     WidgetList children;
+    Pixmap pix;
     int nchild;
 
     XtVaGetValues(w, XtNradioData, &info, NULL);
@@ -803,8 +811,17 @@ deletePatternCB(Widget mb, Widget w)
 	return;
     }
     if (w == info->li->edit) info->li->edit = 0;
+    XtVaGetValues(info->li->fat, XtNpattern, &pix, NULL);
+    if (pix == info->pixmap) {
+       XtVaSetValues(info->li->fat, XtNfillRule, FillSolid, NULL);
+    }
+    XtVaGetValues(info->li->fat, XtNlinePattern, &pix, NULL);
+    if (pix == info->pixmap) {
+       XtVaSetValues(info->li->fat, XtNlineFillRule, FillSolid, NULL);
+    }   
+    XtUnmanageChild(info->widget);   
     XtDestroyWidget(info->widget);
-    patternResized(w, info->li, NULL, 0);   
+    patternResized(w, info->li, NULL, 0);
 }
 
 static void
@@ -1113,6 +1130,7 @@ colorSelectCallback(Widget w, LocalInfo * l, XPointer * junk2)
     XtVaGetValues(icon, XtNradioData, &pi, NULL);
     XawToggleSetCurrent(l->patternlist, (XtPointer) pi);
     if (l->paint) AddItemToCanvasPalette(l->paint, p, None);
+    patternResized(l->fat, l, NULL, 0);
 }
 
 static void 
@@ -1875,7 +1893,7 @@ PatternEdit(Widget w, Pixel *pixels, Pixmap *patterns, void *brushes,
 				    NULL);
     info->patternbox = XtVaCreateManagedWidget("patternRack",
 			            boxWidgetClass, info->vport2,
-				    XtNforceBars, True,
+				    XtNforceBars, False,
 			            NULL);
 
     /*
